//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "PJNPOP3.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


//////////////////// Implementation ///////////////////////////////////////////

class CPJNPOP3App : public CWinApp
{
protected:
  BOOL InitInstance() override;
};

#pragma warning(suppress: 26426)
CPJNPOP3App theApp;


BOOL CPJNPOP3App::InitInstance()
{
  //initialise sockets
  if (!AfxSocketInit())
  {
    ATLTRACE(_T("Failed to initialise the Winsock stack\n"));
    return FALSE;
  }

  //try out the POP3 Client class
  CPJNPOP3Connection p3;

  try
  {
    //you should change the parameters address, name, password and possibly port number if using SSL
    //to connect to your own POP3 mailbox
    p3.Connect(_T("mail.someisp.com"), _T("auser"), _T("apassword"), 110, CPJNPOP3Connection::AuthenticationMethod::AUTH_PLAIN);

    p3.Noop();

    int nMails{0};
    int nSize{0};
    p3.Statistics(nMails, nSize);
    ATLTRACE(_T("%d mail(s) are waiting, total size:%d\n"), nMails, nSize);

    if (nMails)
    {
      for (int j{1}; j<=nMails; j++)
      {
        CPJNPOP3Buffer message;
        const DWORD dwSize{p3.GetMessageSize(j)};
        UNREFERENCED_PARAMETER(dwSize);
        ATLTRACE(_T("Retrieving message %d\n"), j);
        p3.Retrieve(j, message);

        //Save the mail to a file on disk
        ATLTRACE(_T("Saving message %d\n"), j);
        CFile f;
        CString sSave;
        sSave.Format(_T("message%d.eml"), j);
        if (f.Open(sSave, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite))
        {
          const BYTE* pMessage{message.data()};
          ASSERT(pMessage != nullptr);
#pragma warning(suppress: 26472)
          f.Write(pMessage, static_cast<UINT>(message.size()));
        }
        else
          ATLTRACE(_T("Failed to write message to file\n"));

        f.Close();
      }

      for (int j{1}; j<=nMails; j++)
      {
        CPJNPOP3Buffer message;
        const DWORD dwSize{p3.GetMessageSize(j)};
        UNREFERENCED_PARAMETER(dwSize);
        ATLTRACE(_T("Retrieving message header %d\n"), j);
        p3.GetMessageHeader(j, message);

        //Same the mail to a file on disk
        ATLTRACE(_T("Saving message header %d\n"), j);
        CFile f;
        CString sSave;
        sSave.Format(_T("message%d.eml.hdr"), j);
        if (f.Open(sSave, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite))
        {
          const BYTE* pMessage{message.data()};
          ASSERT(pMessage != nullptr);
#pragma warning(suppress: 26472)
          f.Write(pMessage, static_cast<UINT>(message.size()));
        }
        else
        {
          ATLTRACE(_T("Failed to write message header to file\n"));
        }
      }

      ATLTRACE(_T("Getting messages ids\n"));
      for (int i=1; i<=nMails; i++)
      {
        CPJNPOP3String sID(p3.GetMessageID(i));
        ATLTRACE(_T("Message#:%d, ID:%s\n"), i, sID.c_str());
        const int nMessageIndex{p3.FindMessageID(sID.c_str())};
        UNREFERENCED_PARAMETER(nMessageIndex);
      }

      ATLTRACE(_T("Deleting message 1 (Just a test as we will issue a RSET command later)\n"));
      p3.Delete(1);

      ATLTRACE(_T("Attempting to delete a non-existent message, message#:%d\n"), nMails + 1);
      try
      {
        p3.Delete(nMails + 1);
      }
#pragma warning(suppress: 26496)
      catch(CPJNPOP3Exception& e)
      {
      #ifndef _DEBUG
        UNREFERENCED_PARAMETER(e);
      #endif //#ifndef _DEBUG
        ATLTRACE(_T("Failed to delete a non-existent message, Good!!!, Error:%08X\n"), e.m_hr);
      }

      ATLTRACE(_T("Issuing the RSET command to undo the message delete above\n"));
      p3.Reset();
    }
    else
    {
      ATLTRACE(_T("No mails waiting for you at the POP3 server\n"));
    }

    p3.Disconnect();
  }
#pragma warning(suppress: 26496)
  catch(CPJNPOP3Exception& e)
  {
  #ifndef _DEBUG
    UNREFERENCED_PARAMETER(e);
  #endif //#ifndef _DEBUG
    ATL::CAtlString sMessage;
    e.GetErrorMessage(sMessage.GetBuffer(4096), 4096);
    sMessage.ReleaseBuffer();
    ATLTRACE(_T("An unexpected POP3 exception has occurred, Error:%08X, %s, Last Response:%s\n"), e.m_hr, sMessage.GetString(), e.m_sLastResponse.c_str());
  }

  return FALSE;
}
